Package: MetaEntropy
Title: Functional Shannon Entropy for Virome Mutational Analysis
Version: 1.4
Date: 2026-08-31
Authors@R: c( person("Leandro Roberto", "Jones", email = "lrj000@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-5877-4194")),
    person("Julieta Marina", "Manrique", role = c("aut"), comment = c(ORCID = "0000-0001-8712-6666")))
Description: Estimates Shannon entropy, per gene and per genomic position,
    associated with non-synonymous mutation frequencies in viral populations,
    such as wastewater samples. The package uses codon translations for
    functional insights. Each amino acid can be treated as an individual state,
    resulting in a 20-state entropy computation, or grouped into one of six
    physicochemical classes, adding further functional context. Provides
    normalized values (0-1 scale) to facilitate the direct comparison of
    different genomic positions or total functional entropy across multiple
    metagenomes. Designed to analyze mutational data using tabular 'Single
    Nucleotide Variant' (SNV) frequency tables generated by variant callers
    (e.g., 'iVar' or 'LoFreq'), operating independently of consensus sequence
    estimation and multiple sequence alignment.
Encoding: UTF-8
Depends: R (>= 4.1.0)
LazyData: true
Suggests: rmarkdown, tidyr
Imports: ggplot2, patchwork, beeswarm, ggbeeswarm, knitr, dplyr,
        grDevices, rlang
VignetteBuilder: knitr
License: MIT + file LICENSE
Config/roxygen2/version: 8.1.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-09-02 14:31:17 UTC; leandro
Author: Leandro Roberto Jones [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5877-4194>),
  Julieta Marina Manrique [aut] (ORCID:
    <https://orcid.org/0000-0001-8712-6666>)
Maintainer: Leandro Roberto Jones <lrj000@gmail.com>
Repository: CRAN
Date/Publication: 2026-09-03 14:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,entropyProfile)
S3method(plot,entropyProfile)
S3method(print,profileSummary)
S3method(print,tidyMutations)
S3method(summary,entropyProfile)
export(assessHotSpot)
export(entropyProfile)
export(getEntropySignature)
export(showMutations)
importFrom(patchwork,wrap_plots)

#' Build a structure representing amino acid categories. 
#' 
#' The function is used internally by \code{\link{fillPosition}}, which in
#' turn is an auxiliary function of \code{\link{getEntropySignature}}.
#' It creates a list with one element for each amino acid category, named
#' according to the categories used (e.g., "aliphatic", "aromatic", etc.).
#' Each element contains a set of amino acids identified by one-letter codes.
#' The list also includes an element containing an empty numeric vector, whose
#' names correspond to the labels of each category.
#' This vector is to be populated with the frequency of each category at a
#' given genomic position, by the `fillPosition` function.
#'
#' @param categories A character string. Similar to the \code{categories}
#'                   parameter of \code{\link{getEntropySignature}}.
#'
#' @return A list with one element (\code{character} vector) for amino acid
#'         category and an element (empty named \code{numeric} vector) to be
#'         loaded with the frequency in the metagenome of each amino acid category.
#'
#' @seealso \code{\link{getEntropySignature}}.
#'
createStorage <- function(categories){ # "sensitive"/"robust"
	switch(categories,
	       robust = catsAndFreqs <- list(
						aliphatic = c("A","V","L","I","M","C"),
						aromatic = c("F","W","Y","H"),
						polar = c("S","T","N","Q"),
						positive = c("K","R"),
						negative = c("D","E"),
						special = c("G","P"),
						STOP = "*",
						frequencies = stats::setNames(# create an empty, named object
								       numeric(length = 7),# the object
								       c(
									 "aliphatic", "aromatic", "polar", "positive", "negative", "special", "STOP"# the names
								        )
						                      )
	                                       ),
	       sensitive = catsAndFreqs <- list(
					     phe = "F",
					     trp = "W",
					     tyr = "Y",
					     met = "M",
					     leu = "L",
					     val = "V",
					     ile = "I",
					     arg = "R",
					     lys = "K",
					     gln = "Q",
					     glu = "E",
					     asp = "D",
					     asn = "N",
					     ser = "S",
					     gly = "G",
					     ala = "A",
					     thr = "T",
					     pro = "P",
					     cys = "C",
					     his = "H",
					     STOP = "*",
					     frequencies = stats::setNames(
								    numeric(length = 21),
								    c(
								      "phe", "trp", "tyr", "met", "leu", "val", "ile", "arg", "lys", "gln", "glu", "asp", "asn", "ser", "gly", "ala", "thr", "pro", "cys", "his", "STOP")# the names
						                   )
			                    )
	)
	return(catsAndFreqs)
}

#' Translate amino acid frequencies into category frequencies
#'
#' The function is used internally by \code{\link{getEntropySignature}}.
#' It creates a storage list by \code{createStorage}, and loads on it the
#' frequency of each amino acid category based on the data contained in a data
#' frame passed to the function (\code{positionsSummary} parameter).
#'
#' @param positionSummary A data frame created by \code{\link{createPositionSummary}}.
#' @param categories A character string indicating which category scheme to
#'                   use. Similar to the \code{categories} parameter of
#'                   \code{\link{getEntropySignature}}.
#'
#' @return A list with information on the frequencies of each amino acid
#'         category observed in a virome in a specific locus. The list contains
#'         a \code{character} vector for each amino acid category, and a named
#'         \code{numeric} vector containing the frequency of each category in
#'         the metagenome. 
#'
#' @seealso \code{\link{getEntropySignature}}.
#'
fillPosition <- function(positionSummary, categories){
	aminoAcids <- positionSummary$aminoAcids
	frequencies <- positionSummary$frequencies
	#
	variantPosition <- createStorage(categories)
	#
	for(aminoAcid in 1:length(aminoAcids)){
		for(categoria in names(variantPosition[names(variantPosition) != "frequencies"])){
			if(!is.na(match(aminoAcids[aminoAcid], variantPosition[[categoria]]))){
				variantPosition$frequencies[categoria] <- variantPosition$frequencies[categoria] + frequencies[aminoAcid]
			}
		}
	}
	return(variantPosition)
}

#' CDS topology and length of Wuhan-Hu-1 reference strain
#' 
#' This type of data can be obtained from \code{.gff} (General Feature Format)
#' files using applications such as the \code{rtracklayer} package, or manually
#' from the corresponding entry in the GenBank database.
#'
#' @source
#' Nucleotide \[Internet\]. Bethesda (MD): National Library of Medicine (US),
#' National Center for Biotechnology Information; \[1988\] – . Accession No.
#' MN908947.3, Severe acute respiratory syndrome coronavirus 2 isolate Wuhan-Hu-1, complete genome.
#' Available from: \url{https://www.ncbi.nlm.nih.gov/nuccore/MN908947.3}.
#' 
"mn908947.3"

#' Evaluates Entropy Hotspot
#'
#' Graphical and formal analyses of contiguous amino acids.
#'
#' @param profile An object of class \code{entropyProfile}.
#' @param boundaries Numeric vector with the first and last genomic positions
#'        of the region to be evaluated. To be set interactively if not
#'        provided.
#' @param chartType Chart type; either "boxplot", "stripchart" or "swarm".
#'
#' @return \code{htest} object. This function is called primarily for its side
#'          effects.
#'
#' @details The query stretch (*e.g.* a protein domain with neutralizing
#'          epitopes) is compared against the full set of proteins. Hot spot
#'          boundaries should be indicated relative to the reference genome
#'          used in variant calling.
#'
#' @seealso \code{\link{getEntropySignature}}.
#'
#' @examples
#' omicron <- getEntropySignature(wWater[wWater$wave == "third", ])
#' 
#' # Entrpy hotspot at SARS-CoV-2 receptor binding domain
#' assessHotSpot(omicron, c(22517, 23186), chartType = "swarm")
#'
#' @export
#'
assessHotSpot <- function(profile, boundaries, chartType = "boxplot"){
	#
	correctParameters <- c(perfil = FALSE, boundaries = FALSE)
	while(!correctParameters["perfil"] & !correctParameters["boundaries"]){
		# delimit boundaries:
		if(missing(boundaries)){
			while(!correctParameters["boundaries"]){
				# read boundaries
				boundaries <- readline(prompt = "Enter boundaries separated by a coma: ")
				# inspect input
				incorrectInput <- tryCatch(
							   expr = {
								   boundaries <- as.numeric(unlist(strsplit(boundaries, ",")))# warns if non numeric
								   list(boundaries = boundaries, warned = FALSE)
							   }, warning = function(w){
								   list(boundaries = NULL, warned = TRUE)
							      }
				)
				# check if valid
				if(isTRUE(incorrectInput$warned)){
					message("Boundaries must be numbers separated by commas (e.g. \"22518, 23185\").")
				}
				else{
					correctParameters["boundaries"] <- TRUE
				}
			}
		}
		# check profile
		if(!(inherits(profile, "entropyProfile"))){
			stop("The \"profile\" parameter must be an object of class \"entropyProfile\"")
		}
		else{
			correctParameters["perfil"] <- TRUE
		}
	}
	#
	isQuery <- function(positions){
		positions >= boundaries[1] & positions <= boundaries[2]
	}
	is_hot_spot <- factor(length(profile$Entropy$positions), levels = c("no", "yes"))
	is_hot_spot[isQuery(profile$Entropy$position)] <- "yes"
	is_hot_spot[!isQuery(profile$Entropy$position)] <- "no"
	#
	# graphical analysis 
	switch(chartType,
	       boxplot = {
		       graphics::boxplot(profile$Entropy$entropy ~ is_hot_spot,
					 ylab = "entropy",
					 xlab = "hot spot"
		       )
	       },
	       stripchart = {
		       df <- cbind(profile$Entropy, data.frame(hot_spot = is_hot_spot))
		       graphics::stripchart(entropy ~ is_hot_spot, data = df,
					    vertical = TRUE,
					    method = "jitter",
					    pch = 19,
					    xlab = " hot spot"
		       )
	       },
	       swarm ={
		       beeswarm::beeswarm(profile$Entropy$entropy ~ is_hot_spot,
					  pch = 19,
					  ylab = "entropy",
					  xlab = "hot spot"
		       )
	       }
	)
	#
	# formal analysis
	entropy  <- profile$Entropy$entropy
	hot_spot <- is_hot_spot
	stats::kruskal.test(entropy ~ hot_spot)
}

#' Summarize mutations
#'
#' Displays SNVs, and corresponding protein mutations, at specific genomic
#' positions.
#'
#' @param profile  An object of class \code{entropyProfile}.
#' @param positions A vector with genome positions relative to the reference
#'        genome.
#'
#' @return An object of class \code{c("tidyMutations", "data.frame")},
#'         containing summary information about user-supplied genomic
#'         positions. This information includes the mutations themselves
#'         relative to the reference genome, their positions within it, and the
#'         corresponding abundances in the virome. Intended to be displayed by
#'         \code{print.tidyMutations}.
#'
#' @details The user provides a list of genome positions and the function
#'          prints the mutations associated with them.
#'          The output format is "ref_res###alt_res /
#'          protein:ref_res###alt_res", where ref_res is the residue (eiter
#'          nucleotide or aminoacid) in the reference strain, alt_res is the
#'          alternative residue in the metagenome, "###" is the position
#'          (either nucleotide or aminoacid) where the mutation was observed,
#'          and "protein" is the name of the affected protein.
#'
#' @seealso \code{\link{getEntropySignature}}.
#'
#' @examples
#' 
#' # High entropy at the RBD in Omicron lineages
#' omicron <- getEntropySignature(wWater[wWater$wave == "third", ])
#' plot(omicron, chartType="stripchart")
#'
#' # Identify the high-entropy positions
#' omicron$Entropy$position[ omicron$Entropy$entropy > 0.3 ]
#' #[1] 22882 22898 22917 23013 23040 23048 23055 23063
#' 
#' # Get a descriptive table
#' showMutations(omicron, c(22882, 22898, 22917, 23013, 23040, 23048, 23055, 23063))
#'
#'
#' @export
#
showMutations <- function(profile, positions){
	#
	if(!(inherits(profile, "entropyProfile"))){
		stop("The \"profile\" parameter must be an object of class \"entropyProfile\"")
	}
	if(!is.numeric(positions) | !is.vector(positions)){
		stop("The \"positions\" parameter must be a numeric vector (e.g. c(22882, 22898, 22917))")
	}
	#
	# See if "positions" have mutations
	if(any(positions %in% profile$SNVs$position)){ # at least one position displays mutations:
		# See if there are invariant positions also
		if(length(positions)# number of positions passed to the function
		   >
		   sum(positions %in% profile$SNVs$position)){# number of such positions having mutations
			withoutRecords <- positions[ !(positions %in% profile$SNVs$position) ]
			message(paste("No mutations recorded at:", paste(withoutRecords, collapse = ", "), "\n"))
			positions <- positions[ positions %in% profile$SNVs$position ]
		}
	}
	else{
		message(paste("No mutations recorded at:", paste(positions, collapse = ", "), "; returning NULL"))
		return(NULL)
	}
	#
	# Account for multi-haplotype positions
	snv_rows <- which(profile$SNVs$position %in% positions)
	#  
	#
	df <- data.frame(snv = paste0(profile$SNVs$ref[ snv_rows ],
					   profile$SNVs$position[ snv_rows ],
					   profile$SNVs$alt[ snv_rows ]
					   ),
			 phenotype = paste0(profile$SNVs$protein[ snv_rows ],
					    ":",
					    profile$SNVs$ref_aa[ snv_rows ],
					    profile$SNVs$aa_position[ snv_rows ],
					    profile$SNVs$alt_aa[ snv_rows ]
					    ),
			 abundance = round(profile$SNVs$alt_aa_freq[ snv_rows ] * 100)
	)
	#
	# Set class for pretty console output ("print.tydyMutations.R")
	class(df) <- c("tidyMutations", class(df))
	#
	return(df)
	#
}

#' Print method for  \code{tidyMutations} objects
#'
#' This function formats and prints compact mutation summaries
#' (\code{tidyMutations} objects) created by showMutations(), on the console.
#'
#' @param x An object of class \code{tidyMutations} created by
#'          \code{\link{showMutations}}.
#' @param ... Additional arguments passed to the function.
#' 
#' @return Invisibly returns \code{NULL}. Called for side effect.
#'
#' @method print tidyMutations
#' 
#' @export
#' 
#'
print.tidyMutations <- function(x, ...) {
	knitr::kable(x,
		     col.names = c(names(x)[1:2], "abundance (%)"),
		     align = "c") |> cat(sep = "\n")
	invisible(NULL)
}

#' Plot entropy signatures
#'
#' Creates entropy charts along a genome.
#'
#' @param x  Object of class \code{entropyProfile}.
#' @param chartType Whether to graph per-protein summaries ("bp"), per-protein
#'                  stripcharts ("stripchart" / "swarm"), or position-wise
#'                  entropy ("entroScan").
#' @param ... Additional arguments passed to the function.
#'
#' @return Unrendered \code{gg/ggplot} object produced by \code{ggplot2}. This
#'                    function is primarily called for its side effects.
#'
#' @importFrom patchwork wrap_plots
#'
#' @examples
#' ancestral <- getEntropySignature(wWater[wWater$wave == "first", ])
#' omicron <- getEntropySignature(wWater[wWater$wave == "third", ])
#' 
#' # Enhanced Spike entropy plus pervasive negative selection in Omicron
#' # sublineages
#' anc_plot <- plot(ancestral, chartType = "stripchart")
#' omi_plot <- plot(omicron, chartType = "stripchart")
#' patchwork::wrap_plots(anc_plot/omi_plot)
#'
#'
#' @export
#'
#' @method plot entropyProfile
#'
plot.entropyProfile <- function(x, chartType = "bp", ...){
	switch(chartType,
	       bp = {ggplot2::ggplot(as.data.frame(x), ggplot2::aes(x = protein, y = entropy)) +
		            ggplot2::geom_boxplot() +
			    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust = 1))
	       },
	       stripchart = {ggplot2::ggplot(as.data.frame(x), ggplot2::aes(x = protein, y = entropy, col = protein)) +
		       ggplot2::geom_jitter(width = 0.2) +
		       ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust = 1))

	       },
	       entroScan = {ggplot2::ggplot(as.data.frame(x), ggplot2::aes(x = position, y = entropy, col = protein)) +
		       ggplot2::geom_point() +
		       ggplot2::labs(x = "genomic position") +
		       ggplot2::coord_cartesian(xlim = c(1, x$Genome$Length)) +
		       ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust = 1))
	       },
	       swarm = {ggplot2::ggplot(as.data.frame(x), ggplot2::aes(x = protein, y = entropy, col = protein)) +
		       ggbeeswarm::geom_beeswarm() +
		       ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust = 1))
	       }
	)
	
}

#' Summarize entropy profile
#'
#' Prints a report about an entropy profile (an object of class
#' "entropyProfile").
#'
#' @param object An object of class \code{entropyProfile}.
#' @param ...    Other parameters passed to the function.
#'
#' @rdname summary.entropyProfile
#' 
#' @return An object of class \code{c("profileSummary", "list")} summarizing
#'         an entropy profile. Intended to be displayed via
#'         \code{print.profileSummary}.
#' 
#' @export
#'
#' @method summary entropyProfile
#
summary.entropyProfile <- function(object, ...){
	#
	feature <- c("allCDS", as.character(object$Mutations$protein))
	df <- data.frame(feature	 = feature,
			 min		= numeric(length(feature)),
			 firstQu	= numeric(length(feature)),
			 median		= numeric(length(feature)),
			 mean		= numeric(length(feature)),
			 thirdQu	= numeric(length(feature)),
			 max		= numeric(length(feature)),
			 SNV_Kb		= numeric(length(feature)),
			 nonSyn_Kb	= numeric(length(feature))
	)
	#
	# generic descriptive statistics
	for(df_row in 1:dim(df)[1]){
		if(df$feature[df_row] == "allCDS"){
			df[df_row, 2:7] <- as.numeric(summary(object$Entropy$entropy))
		}
		else{
			df[df_row, 2:7] <- as.numeric(summary(object$Entropy$entropy[object$Entropy$protein == df$feature[df_row]]))
		}
	}
	#
	# Sumarize mutations
	# total (all CDS; syn[1]) and per-protein synonymous mutations
	syn <- c(sum(object$Mutations$syn), object$Mutations$syn)
	# the same for non-synonymous mutations
	nonSyn <- c(sum(object$Mutations$nonSyn), object$Mutations$nonSyn)
	SNVs <- syn + nonSyn
	# All CDS and per-protein lengths
	cdsLengths <- c(sum(object$Mutations$cdsLength), object$Mutations$cdsLength)
	# mutational densities (all CDSs plus every CDS)
	norm_SNVs <- 1000 * SNVs/cdsLengths
	# non-synonymous mutational densities
	norm_nonSyn <- 1000 * nonSyn/cdsLengths
	df$SNV_Kb <- norm_SNVs
	df$nonSyn_Kb <- norm_nonSyn
	#
	lst <- list(summary = df,
			metagenome = deparse(substitute(object)),
			SNVnum = sum(SNVs),
			synNum = sum(syn),
			nonSynNum = sum(nonSyn)
	)
	#
	class(lst) <- c("profileSummary", class(lst))
	#
	return(lst)
	#
}

#' Print method for  \code{profileSummary} objects
#'
#' This function formats and prints compact entropy profile summaries
#' (\code{profileSummary} objects) created by \code{summary.entropyProfile()},
#' on the console.
#'
#' @param x An object of class \code{profileSummary} created by
#'          \code{\link{summary.entropyProfile}}.
#' @param ... Additional arguments passed to the function.
#' 
#' @return Invisibly returns \code{NULL}. This function is used for its side
#'         effect.
#' 
#' @method print profileSummary
#' 
#' @export
#' 
#
print.profileSummary <- function(x, ...) {
	gsub("Table: ",
	     "",
	     knitr::kable(x$summary,
			  col.names = c("Feature",
					"Min.",
					"1st Qu.",
					"Median",
					"Mean",
					"3rd Qu.",
					"Max.",
					"SNVs/Kb",
					"non-Syn/Kb"
					),
			  caption = paste0("\n'",
					   paste0(toupper(substr(x$metagenome, 1, 1)),
						  tolower(substr(x$metagenome, 2, nchar(x$metagenome)))
						  ),
					   "' entropy signature.\n\nSummary of entropy and mutations by protein:"),
			  align = "c",
			  digits = 2
	     )

	) |> cat(sep = "\n")
	#
	invisible(NULL)
}

#' Data from first and third COVID-19 waves in Trelew \url{http://tools.wmflabs.org/geohack/geohack.php?language=es&pagename=Trelew&params=-43.253333333333_N_-65.309444444444_E_type:city}
#' 
#' 
#' SNVs inferred from Illumina (2 x 150) sequences from pooled ultra-pure virus
#' concentrates representative of the 1st and 3rd COVID-19 waves in Trelew.
#' Reads were mapped against the Wuhan-Wu-1 reference genome (MN908947.3) by
#' \emph{bwa}, and variants were called with \emph{iVar} with a 3% frequency
#' cutoff for minor variants.
#' First wave cases were caused by ancestral strains whereas third wave cases
#' were mainly due to highly human-adapted Omicron sublineages.
#'
#' @source
#'   Manrique, Julieta Marina, and Leandro Roberto Jones. 2026. Genomic
#'   surveillance of SARS-CoV-2: a resource-efficient wastewater-based
#'   workflow. Molecular Biology Reports 53 : 995.
#' 
"wWater"

#' Summarize variants and frequencies at a genome position
#'
#' This function is used internally by \code{getEntropySignature()}.
#' It creates a vector (\code{aminoAcids}) listing the amino acids
#' observed in a virome at a particular position under analysis, including the
#' reference amino acid, another vector (\code{frequencies}) with the
#' corresponding frequencies, and returns them combined in a data frame.
#'
#' @param variants A data frame, similar to the \code{polymorphisms} argument
#'                 of \code{\link{getEntropySignature}}, but containing
#'                 information on a single genome position.
#' @param ref_aa Name of the column that carries reference amino acids.
#' @param alt_aa Name of the column carrying alternative amino acids observed
#'               in the metagenome.
#' @param alt_aa_freq Name of the column giving the frequencies of alternative
#'                 amino acids.
#'
#' @return A \code{data frame} describing the variability (different amino
#'         acids its frequencies) observed at a specific locus.
#'
#' @seealso \code{\link{getEntropySignature}}.
#'
#
createPositionSummary <- function(variants, ref_aa, alt_aa, alt_aa_freq){
	aminoAcids <- character(length = dim(variants)[1] + 1)# +1: space for the variant equal to ref_aa
	aminoAcids[1] <- variants[,ref_aa][1]
	frequencies <- numeric(length = dim(variants)[1] + 1)
	# load alternative variant/s and their frequencies
	for(variante in 1:length(variants[,alt_aa])){# 1: (i.e., not 2:) used to remind variant 1 is the same as the ref. aa
		aminoAcids[variante + 1] <- variants[,alt_aa][variante]
		frequencies[variante + 1] <- variants[,alt_aa_freq][variante]
	}
	frequencies[1] <- 1 - sum(frequencies[frequencies != 0])
	return(data.frame(aminoAcids = aminoAcids, frequencies = frequencies))
}

#' Graphically compare entropy profiles
#'
#' This function prints heatmaps of multiple entropy profiles
#' (\code{entropyProfile} objects).
#'
#'
#' @param ... Unquoted \code{entropyProfile} data objects to be compared.
#' @param keepInvariant Logical; whether to retain positions where entropy is
#'   zero. Defaults to \code{FALSE}.
#' @param showAllPeptides Logical; whether the heatmap should show all peptides
#'   encoded by the virus, regardless of whether they have undergone mutations.
#'   Defaults to \code{TRUE}.
#' 
#' @return A \code{ggplot} object representing the heatmap.
#'
#' @importFrom dplyr summarize group_by
#' @importFrom grDevices rgb
#' @importFrom rlang enquos
#'
#' @examples
#' firstWave <- wWater[ wWater$wave == "first", ]
#' thirdWave <- wWater[ wWater$wave == "third", ]
#' # Filter out minor variants whose genotype matches that of the reference genome
#' firstWave <- firstWave[ firstWave$alt_aa_freq <= 0.97, ]
#' thirdWave <- thirdWave[ thirdWave$alt_aa_freq <= 0.97, ]
#' ancestral <- getEntropySignature(firstWave)
#' omicron <-  getEntropySignature(thirdWave)
#' 
#' # High entropies sustained over time in the spike protein, and signals
#' # compatible with pervasive negative selection in Omicron sublineages:
#' heatmap_entropyProfiles(ancestral, omicron)
#' 
#' @export
#
heatmap_entropyProfiles <- function(..., keepInvariant = TRUE, showAllPeptides = TRUE){
	#
	# Capture arguments passed into the ellipsis
	quos <- rlang::enquos(...)
	quos_names <- names(quos) # Capture the argument/list names
	# Validate
	valid_profiles <- list()
	for (i in seq_along(quos)) {
		# Determine profile names in a way compatible with lists,
		# do.call(), and !!!: use the list/argument name if it exists,
		# otherwise fall back to the expression label (for manual inputs)
		if (!is.null(quos_names) && quos_names[i] != "") {
			# For heatmap_entropyProfiles(!!!profiles),
			# quos_names[i] will cleanly pull, e.g. "c1_heart",
			# "c5_liver", etc. (see "intraHostVariants" vignette),
			# directly from the "profiles" list keys.
			expr_name <- quos_names[i]
		} else {
			# For manual inputs like heatmap_entropyProfiles(c1_heart),
			# quos_names will be empty, so the function uses
			# as_label() tu pull "c1_heart".
			expr_name <- rlang::as_label(quos[[i]])
		}
		# Evaluate the expression
		current_obj <- tryCatch(rlang::eval_tidy(quos[[i]]), error = function(e) NULL)
		if (is.null(current_obj)) {
			warning(paste0("Object '", expr_name, "' not found. Skipping."))
			next
		}
		# Perform class check
		obj_class <- class(current_obj)[1]
		if (obj_class == "entropyProfile") {
			valid_profiles[[expr_name]] <- current_obj
		} else {
			warning(paste0("Object '", expr_name, "' is a ", obj_class, ", not entropyProfile. Skipping."))
		}
	}
	#
	# Extract and format data
	profiles <- data.frame()
	for(profile in names(valid_profiles)){
		thisProfile <- valid_profiles[[ profile ]]$Entropy |>
			dplyr::group_by(protein) |>
			dplyr::summarize(entropy = mean(entropy), .groups = "drop") |>
			as.data.frame()
		thisProfile[["stratum"]] <- profile
		profiles <- rbind(profiles, thisProfile)
	}
	# Process
	# Check if zero entropy positions are to be displayed
	if (!keepInvariant){
		profiles <- profiles[profiles$entropy > 0, ]
		minColor <- grDevices::rgb(1, 0.95, 0.95) # very faint red (lowest entropy)
		maxColor = grDevices::rgb(1, 0, 0)
	}
	else{
		minColor <- grDevices::rgb(1, 1, 1) # white (entropy = 0)
		maxColor = grDevices::rgb(1, 0, 0)
	}
	# See if all peptides are to be represented in the heatmap and ensure
	# they are plotted in genome order:
	# Extract ALL proteins from ALL valid profiles. Note that this
	# allows comparison of profiles generated using different ways of
	# encoding the genome (e.g., one with nsp12a and nsp12b, and another
	# with nsp12; see "intraHostVariants" vignette)
	all_reference_peptides <- unique(unlist(lapply(valid_profiles, function(x) x$Genome$CDS$protein)))
	if(showAllPeptides){
		# Use the combined reference list safely
		maturePeptides <- all_reference_peptides
		profiles$protein <- factor(profiles$protein,
					   levels = maturePeptides
		)
	}
	else{
		# Intersect with the combined reference list safely
		maturePeptides <- intersect(all_reference_peptides, profiles$protein)
		profiles$protein <- factor(profiles$protein,
					   levels = maturePeptides)
	}
	#
	# Plot
	return(
	      ggplot2::ggplot(profiles, ggplot2::aes(x = protein, y = stratum, fill = entropy)) +
		      ggplot2::geom_tile(color = "black", linewidth = 0.5) +
		      ggplot2::scale_fill_continuous(low = minColor, high = maxColor, name = "Mean\nEntropy") +
		      ggplot2::scale_x_discrete(drop = FALSE) +
		      ggplot2::theme_minimal() +
		      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust = 1))
	)
	#
}

#' Create (empty) object of class "entropyProfile"
#'
#' This function is intended primarily for internal use by
#' \code{\link{getEntropySignature}}.
#'
#' @param polymorphisms  A data frame. Please see Details and Examples in
#'        documentation for \code{\link{getEntropySignature}}.
#' @param position Name of the \code{polymorphisms}'s column that indicates SNV
#'                 locations in the genome.
#' @param linkage Information on linked positions.
#' @param ref Column name with reference bases.
#' @param alt Column name with the alternative bases observed in the
#'            metagenome.
#' @param protein Name of the column carrying protein names.
#' @param aa_position Name of the column that indicates the protein positions
#'                    of the mutated amino acids.
#' @param ref_aa Name of the column that carries the reference amino acids.
#' @param alt_aa Name of the column carrying alternative amino acids observed
#'               in the metagenome.
#' @param alt_aa_freq Name of the column giving the frequencies of alternative
#'                 amino acids in the metagenome.
#' @param entropies \code{NA_REAL_} (double numeric/real vector to hold entropy
#'                  values).
#' @param genome A list providing CDS data and length of the reference genome.
#'
#' @details
#' The documentation for \code{\link{getEntropySignature}} details the type of
#' input needed to create a profile. \code{entropyProfile} uses the same parameters as
#' \code{getEntropySignature}, with the exception of \code{categories} and
#' \code{entropies}.
#'
#' @return An (empty) object of class \code{entropyProfile}.
#'
#' @seealso \code{\link{getEntropySignature}}.
#'
entropyProfile <- function(polymorphisms,
			   position = "position",
			   linkage = "linkage",
			   ref = "ref",
			   alt = "alt",
			   protein = "protein",
			   aa_position = "aa_position",
			   ref_aa = "ref_aa",
			   alt_aa = "alt_aa",
			   alt_aa_freq = "alt_aa_freq",
			   entropies = NA_real_,
			   genome = mn908947.3){
	#
	# preserve original data (some point mutions will not be mirrored in
	# the entropy profile due to linkage)
	snvs <- polymorphisms
	# standardize labels:
	names(snvs)[ names(snvs) == position ] <- "position"
	names(snvs)[ names(snvs) == linkage ] <- "linkage"
	names(snvs)[ names(snvs) == ref ] <- "ref"
	names(snvs)[ names(snvs) == alt ] <- "alt"
	names(snvs)[ names(snvs) == protein ] <- "protein"
	names(snvs)[ names(snvs) == aa_position ] <- "aa_position"
	names(snvs)[ names(snvs) == ref_aa ] <- "ref_aa"
	names(snvs)[ names(snvs) == alt_aa ] <- "alt_aa"
	names(snvs)[ names(snvs) == alt_aa_freq ] <- "alt_aa_freq"
	#
	# extract coding DNA sequence data (simplify code below)
	cds <- genome$CDS
	#
	# account for linked positions
	linked <- FALSE
	for(snv in 1:dim(polymorphisms)[1]){
		if(is.na(polymorphisms[,linkage][snv])){
		# nothing to do
		}
		else{
			# what positions are linked?
			haplotype_start <- polymorphisms[,position][snv]
			inside <- TRUE
			while(inside){
				haplotype_end <- polymorphisms[ ,linkage][snv]	# e.g.: snv 10 in position 234 linked to snv 11 in position 235
										#    	wave,	position,	linkage
										#     9	xxx,	100,		NA
										#    10	xxx,	234,		235
										#    11	xxx,	235,		234
										#    12	xxx,	583,		NA
										# haplotype_end = x[,linkage][10] = 235
				snv <- snv + 1# move one snv downstream (e.g., snv 11)
				if(polymorphisms[ ,linkage][snv] < polymorphisms[ ,position ][snv]){# Linked to upstream position only
					inside <- FALSE
					snv <- snv + 1# move one snv downstream (e.g. snv 12)
				}
				else{
					# Do nothing (still inside the haplotype)
				}
			}
			# Colapse linked positions; map residue substitution to haplotype_start
			if(haplotype_end - haplotype_start == 1){# 2 codon positions linked
				polymorphisms <- polymorphisms[ -(snv-1),]# e.g. drop row 11
			}
			else{# All 3 positions linked
				polymorphisms <- polymorphisms[ -c(snv-1, snv-2),]
			}
		}
	}
	#
	# Account for ORF superposition (positions are multiplexed internally)
	# Implements "multiplexed" positions by adding decimals, e.g. position
	# 100, becomes 100, 100.1, 100.2, etc. ("pseudo mapping").
	#
	# A "general" example, with "linkage" (not a real example);
	# All three positions in a codon display mutations
	#
	#       position linkage ref alt  protein   codon    ref_aa alt_aa  freq
	# 50    1        2       G   T    X         123      A      B       0.09884
	# 51    1        2       A   A    Y         456      M      N       0.09884
	# 52    2        3       G   T    X         123      A      B       0.09884
	# 53    2        3       C   G    Y         456      M      N       0.12706
	# 54    3        2       A   A    X         123      A      B       0.09884
	# 55    3        2       C   G    Y         456      M      N       0.12706
	#
	# Ergo, have to "displace" either all the positions associated with protein X or all
	# the positions associated with protein Y.
	#
	# 1. See which positions are repeated
	are_duplicate <- duplicated(polymorphisms[ ,position])
	# returns a logic pointing to cells 50 to 55.
	# 
	duplicates <- unique(polymorphisms[ ,position][are_duplicate])
	# returns "c(1, 2, 3)"
	#
	# 2. See which positions are  repeated due to overlaps ("protein" column will be labelled differently).
	for(this_duplicate in duplicates){
		# check if a single or multiple proteins are listed for the repeated position 
		affected_proteins <- polymorphisms[ polymorphisms[ ,position] == this_duplicate, protein]
		# when this_duplicate is 1 affected_proteins is "c(X, Y)"
		n_affected_proteins <- length(unique(affected_proteins))
		# when this_duplicate is 1, n_affected_proteins is "2" 
		if( n_affected_proteins > 1 ){# there is an overlap; multiplex it (true when this_position equals 1, 2, and 3)
			# See how many times the position is repeated
			repetitions <- length(polymorphisms[ polymorphisms[,position] == this_duplicate ,position])
			# equals "2" for positions 1, 2 and 3
			# Calculate displacements
			displacement <- 1/repetitions # e.g. 4 repetitions gives 0.25.; then
			                              # "x" becomes "x", "x.25", "x.50", "x.75" (never reaches x + 1)
			# diplacement is 0.5 for positions 1, 2 and 3
			# multiplex
			toMultiplex <- which( polymorphisms[,position] == this_duplicate )# a numerical vector, indexing
			                                                                  # the positions to be multiplexed
			# equals "c(50, 51)" when this_duplicate is "1"
			for(displaceThis in toMultiplex[-1]){# [-1] remove the first position of the vector
				# loop starts at "52" when this position is "1"
				polymorphisms[,position][displaceThis] <- polymorphisms[,position][displaceThis - 1] + displacement
				# Using "[displaceThis - 1]" keeps adding "displacement" the the previous member of the
				# When this_position equals "1", our example becomes:
				#
				#       position   linkage ref alt  protein   codon    ref_aa alt_aa  freq
				# 50    1          2       G   T    X         123      A      B       0.09884
				# 53    1.5        2       A   A    Y         456      M      N       0.09884
				# 51    2          3       G   T    X         123      A      B       0.09884
				# 54    2          3       C   G    Y         456      M      N       0.12706
				# 52    3          2       A   A    X         123      A      B       0.09884
				# 55    3          2       C   G    Y         456      M      N       0.12706
				#
				# So, we have to multiplex the linked position informing on protein "Y" (position[52]),
				# because in the next round, when this_positions will be "2", it will be set to 2.5.
				#
				# Check if linked
				this_linkage <- polymorphisms[ , linkage][displaceThis]
				if(!is.na(this_linkage)){
					polymorphisms[ , linkage][displaceThis] <- this_linkage + displacement
				}
				# When thisposition is 1 displaceThis is "52", so the above results in
				#
				#       position   linkage   ref alt  protein   codon    ref_aa alt_aa  freq
				# 50    1          2         G   T    X         123      A      B       0.09884
				# 51    1.5        2.5       A   A    Y         456      M      N       0.09884
				# 52    2          3         G   T    X         123      A      B       0.09884
				# 53    2          3         C   G    Y         456      M      N       0.12706
				# 54    3          2         A   A    X         123      A      B       0.09884
				# 55    3          2         C   G    Y         456      M      N       0.12706
				#
				# And after processing all 3 positions:
				#
				#       position   linkage   ref alt  protein   codon    ref_aa alt_aa  freq
				# 50    1          2         G   T    X         123      A      B       0.09884
				# 51    1.5        2.5       A   A    Y         456      M      N       0.09884
				# 52    2          3         G   T    X         123      A      B       0.09884
				# 53    2.5        3.5       C   G    Y         456      M      N       0.12706
				# 54    3          2         A   A    X         123      A      B       0.09884
				# 55    3.5        2.5       C   G    Y         456      M      N       0.12706
				#
			}
		}
		else{# no overlap, ergo do nothing
		}
	}
	#
	# proteins with records
	proteins <- character()
	for(posicion in unique(polymorphisms[ ,position])){
		proteins <- c(proteins, unique(polymorphisms[ ,protein][polymorphisms[ ,position] == posicion]))
		# unique() is to account for mutiple haplotypes.
		# E.g.:
		# > polymorphisms[ polymorphisms[,position] == 23429 ,]
		#     wave position linkage ref alt protein aa_position ref_aa alt_aa alt_aa_freq
		# 50 first    23429      NA   G   T       S         623      A      S     0.09884
		# 51 first    23429      NA   G   A       S         623      A      T     0.12706
		# These two rows make a joint contribution to the entropy at
		# position 23429, so the profile will contain a single row for
		# position 23429 (ergo a unique protein label, "S").
	}
	# keep order of proteins (most likely genomic order)
	proteins <- factor(proteins, levels = unique(proteins[!is.na(proteins)]))
	#
	#
	# Structure that carries the profile
	perfil <- list(SNVs = snvs,
		       Entropy = data.frame(position = unique(polymorphisms[,position]),# again account for multi-haplotype polymorphisms
					    protein = proteins,
					    entropy = entropies
		       ),
		       Mutations = data.frame(protein = unique(proteins), # proteins with records
					      cdsLength = cds[ which(cds[ , "protein"] %in% unique(proteins)), "end"] - (cds[ which(cds[ , "protein"] %in% unique(proteins)), "start"] - 1),
					      syn = numeric(length = length(unique(proteins))),
					      nonSyn = numeric(length = length(unique(proteins)))
					      ),
		       Genome = genome 
	)
	#
	# Summarize mutation data. ("polymorphisms" has been already processed above for linked haplotypes)
	for(cdsRegion in unique(perfil$Entropy$protein)){
		perfil$Mutations$syn[perfil$Mutations$protein == cdsRegion] <- sum(polymorphisms[ polymorphisms$protein == cdsRegion, "ref_aa"] == polymorphisms[ polymorphisms$protein == cdsRegion, "alt_aa"])
		perfil$Mutations$nonSyn[perfil$Mutations$protein == cdsRegion] <- sum(polymorphisms[ polymorphisms$protein == cdsRegion, "ref_aa"] != polymorphisms[ polymorphisms$protein == cdsRegion, "alt_aa"])
	}
	#
	#
	class(perfil) <- c("entropyProfile", class(perfil))
	#
	return(list(Perfil = perfil,
		    Polymorphisms_multiplexed = polymorphisms
		    )
	)
}

#' Calculates aminoacidic entropy resulting from SNVs present in a specific locus
#'
#' The function is used internally by \code{\link{getEntropySignature}}.
#' It calculates Shannon entropy from categories and frequencies passed to the
#' function.
#'
#' @param variantPosition A list, created  and passed to the function by
#'                        \code{fillPosition}. It contains information on the
#'                        frequencies of each amino acid category observed in a
#'                        virome, as a result of mutations at a given genomic
#'                        position.
#'
#' @return A \code{numeric} value, corresponding to the entropy associated to
#'         the SNVs present at a specific locus.
#'
#' @seealso \code{\link{getEntropySignature}}.
#'
getPosEntropy <- function(variantPosition){
	frecuencias <- variantPosition$frequencies[variantPosition$frequencies > 0]
	entropia <- 0
	for(thisFrequency in frecuencias){
		       		entropia <- entropia + thisFrequency*log(thisFrequency,
										base = length(variantPosition$frequencies))# scale to [0, 1] range.
	       		   }
	return(-entropia)
}

#' SNVs from multiple organs of individuals with postmortem SARS-CoV-2 detection
#' 
#' SNVs were inferred from Illumina (2 x 150) sequences. Reads were aligned to
#' the Wuhan-Hu-1 reference genome (MN908947.3) with \code{BWA}, followed by
#' variant calling using \code{LoFreq} with source quality correction. The VCF
#' file was post-processed with \code{BCFtools} to prevent reference and strand
#' biases and remove positions supported by fewer than 1,000 reads.
#'
#' @docType data
#' @format A data frame with 213 rows and 16 columns.
#'
#' @seealso 
#' See \code{vignette("intraHostVariants", package = "MetaEntropy")} for a 
#' step-by-step tutorial on analyzing this dataset.
#'
#' @source
#' Manrique, J. M., Maffia-Bizzozero, S., Delpino, M. V., Quarleri, J., &
#' Jones, L. R. (2024). Multi-Organ Spread and Intra-Host Diversity of
#' SARS-CoV-2 Support Viral Persistence, Adaptation, and a Mechanism That
#' Increases Evolvability. \emph{Journal of Medical Virology}, 96(12), e70107. 
#' \doi{10.1002/jmv.70107}
#' 
"intraHostVariants"

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----loadPackages, results = "hide"-------------------------------------------
lapply(c("MetaEntropy", "ggplot2", "patchwork", "tidyr"),
       library, character.only = TRUE
)

## ----computeSNVs--------------------------------------------------------------
addmargins(
	   table(intraHostVariants$case,
		 intraHostVariants$organ
		 ),
	   margin = 2
)

## ----widenData, results = "hide"----------------------------------------------
wide_ihv <- tidyr::pivot_wider(intraHostVariants,
		 names_from = organ,
		 values_from = alt_aa_freq,
		 # Use 0 (instead of NA) for undetected
		 # mutations.
		 values_fill = list(alt_aa_freq = 0)
)

## ----doParCoordPlot, fig.width = 4, fig.height = 8, out.width = "50%", results = "hide"----
par(mfcol = c(4,1), mar = c(2, 4, 2, 2) + 0.1)
for(case in sort(unique(wide_ihv$case))){
	thisCase <- t(wide_ihv[wide_ihv$case == case, c(15, 16, 17, 18, 19)])
	matplot(thisCase, type = "b", pch = 19, col = rgb(0, 0, 0, 0.4), lty = 1,
		xaxt = "n", xlab = "",
		main = sub("c", "case ", case),
		ylab = "SNV frequency"
	)
	axis(side = 1, at = 1:5, labels = rownames(thisCase)
	)
}

## ----c18Contrasts-------------------------------------------------------------
intraHostVariants[intraHostVariants$case == "c18" & intraHostVariants$alt_aa_freq > 0.2 ,
                  c(1, 2, 10, 13, 14, 15, 16)]

## ----splitData, results = "hide"----------------------------------------------
strata <- split(intraHostVariants,
                    list(intraHostVariants$organ, intraHostVariants$case),
                    sep = "_", drop = TRUE
)

## ----createGenome, results = "hide"-------------------------------------------
mn908947.3.ihv <- mn908947.3
# Get the nsp12 3' end and assign it to nsp12_end
nsp12_rows <- mn908947.3.ihv$CDS$protein %in% c("nsp12a", "nsp12b")
nsp12_end <- max(mn908947.3.ihv$CDS$end[nsp12_rows])
# Create an entry for nsp12 from nsp12a
mn908947.3.ihv$CDS$protein[mn908947.3.ihv$CDS$protein == "nsp12a"] <- "nsp12"
# Update the 3' end
mn908947.3.ihv$CDS$end[mn908947.3.ihv$CDS$protein == "nsp12"] <- nsp12_end
# dismiss the old nsp12b annotation
mn908947.3.ihv$CDS <- mn908947.3.ihv$CDS[mn908947.3.ihv$CDS$protein != "nsp12b", ]
# tidy up the environment
rm(nsp12_rows, nsp12_end)

## ----computeEntropies, results = "hide"---------------------------------------
profiles <- lapply(strata, function(df) {
			   getEntropySignature(df, position = "POS", ref = "REF", alt = "ALT",
					       genome = mn908947.3.ihv
			   )
})

## ----doHeatMaps, fig.width = 9, fig.height = 7, out.width = "90%", results = "hide"----
heatmap_entropyProfiles(!!!profiles)

## ----doBoxPlots, fig.width = 9, fig.height = 7, out.width = "90%", results = "hide"----
combined_entropy <- lapply(profiles, function(p) p$Entropy) |> do.call(rbind, args = _)
combined_entropy <- cbind(combined_entropy,
                          system = factor(ifelse(grepl("lung", rownames(combined_entropy)),
                                                       "respiratory system", "other"),
                                          levels = c("respiratory system", "other")
                                   )
                    )
# Plot in genomic order
combined_entropy$protein <- factor(
                                   combined_entropy$protein,
                                   levels = mn908947.3.ihv$CDS$protein
)
bp <- ggplot2::ggplot(data = combined_entropy, aes(x = protein, y = entropy)) +
	ggplot2::geom_boxplot(varwidth = T) +
	ggplot2::facet_wrap(~ system, ncol = 1, scales = "free_y") +
	ggplot2::scale_x_discrete(drop = FALSE) +
	ggplot2::theme_bw() +
	ggplot2::theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
bp

#' Coerce \code{entropyProfile} to a Data Frame
#'
#' Function to extract summary information from an \code{entropyProfile}
#' object. This function is internally used for plotting.
#'
#' @param x An object of class \code{entropyProfile}.
#' @param row.names Please see \code{\link{as.data.frame}}.
#' @param optional Please see \code{\link{as.data.frame}}.
#' @param ... Additional arguments passed to the function.
#'
#' @return A \code{data frame} with tabular information on an entropy profile.
#'         This information includes the name of the proteins presenting
#'         mutations, the corresponding genomic positions, and the resulting
#'         entropies in the metagenome.
#'
#' @method as.data.frame entropyProfile
#'
#' @export
#'
as.data.frame.entropyProfile <- function(x, row.names = NULL, optional = FALSE, ...){
	return(data.frame(protein = x$Entropy["protein"],
			  # use trunc() to "demultiplex" overlapped mutations
			  position = trunc(x$Entropy["position"]),
			  entropy = x$Entropy["entropy"]
               )
	)
}

#' Infer Entropy Signature
#'
#' Calculates genome-wide Shannon entropies from SNV data.
#'
#' @param polymorphisms  A data frame. Please see Details and Examples.
#' @param position Name of the \code{polymorphisms}'s column that indicates SNV
#'                 locations in the genome.
#' @param linkage Information on linked positions.
#' @param ref Column name with reference bases.
#' @param alt Column name with the alternative bases observed in the
#'            metagenome.
#' @param protein Name of the column carrying protein names.
#' @param aa_position Name of the column that indicates the protein positions
#'                    of the mutated amino acids.
#' @param ref_aa Name of the column that carries the reference amino acids.
#' @param alt_aa Name of the column carrying alternative amino acids observed
#'               in the metagenome.
#' @param alt_aa_freq Name of the column giving the frequencies of alternative
#'                 amino acids in the metagenome.
#' @param categories  Whether a class per amino acid should be used
#'        ("sensitive") or they should be grouped into aliphatic, aromatic,
#'        polar, positively charged, negatively charged, and special ("robust")
#'        (Mirny and Shakhnovich, 1999).
#' @param genome A list providing CDS data and length of the reference genome.
#'
#' @return An object of class \code{entropyProfile}. It contains a tidy,
#'         summarized version of the SNV table, a \code{data frame} with
#'         information on genome-wide entropy, a \code{data frame} with
#'         information on each CDS and corresponding mutations observed in the
#'         virome, and a \code{list} with CDS data and length of the reference
#'         genome used in variant calling.
#'
#'
#' @details You provide a data frame with SNVs information including reference
#' and alternative aminoacids, their frequencies, and corresponding positions
#' relative to a reference sequence.
#' This type of data can be generated by numerous programs and pipelines.
#' The objective is to assess the biological impact of nonsynonymous
#' variation within a viral population, such as an environmental sample (e.g.
#' wastewater) or a single infection (aka quasisepecies).
#' Entropy is calculated \emph{within} the metagenome and is therefore independent
#' of the reference sequence.
#' Some mutations may be part of a same codon.
#' This is to be indicated in the \code{linkage} column, providing a downstream
#' linked position, or the closest upstream position if there are no downstream
#' positions that are part of the same codon.
#' For example, in the \code{wWater} dataset, mutations T22673C and C22674T are linked
#' to each other and affect codon 371 of the S gene:
#' \tabular{lccccccc}{
#' 	\tab	wave	\tab	position	\tab	linkage	\tab	ref	\tab	alt	\tab	protein	\tab ... \cr
#' ...	\tab		\tab			\tab		\tab		\tab		\tab		\tab     \cr
#' 105	\tab	third	\tab	22599		\tab	NA	\tab	G	\tab	A	\tab	S	\tab ... \cr
#' 106	\tab	third	\tab	22673		\tab	22674	\tab	T	\tab	C	\tab	S	\tab ... \cr
#' 107	\tab	third	\tab	22674		\tab	22673	\tab	C	\tab	T	\tab	S	\tab ... \cr
#' 108	\tab	third	\tab	22679		\tab	NA	\tab	T	\tab	C	\tab	S	\tab ... \cr
#' ...	\tab		\tab			\tab		\tab		\tab     \cr
#' }
#'
#' The \code{genome} parameter is a list that provides data on the topology of
#' protein-coding regions in the genome and its length, used internally
#' primarily for graphical and summary purposes.
#' The package provides an example (\code{\link{mn908947.3}}) of how  this
#' information is to be organized.
#'
#' @references
#'   Mirny and Shakhnovich, 1999. J Mol Biol 291:177-196.
#'   \doi{10.1006/jmbi.1999.2911}.
#'
#'   Shannon, 1948. Bell System Technical Journal, 27:379-423.
#'   \doi{10.1002/j.1538-7305.1948.tb01338.x}.
#'
#' @examples
#' 
#' # Entropy across the genome in ancestral lineages
#' ancestral <- getEntropySignature(wWater[wWater$wave == "first", ], categories = "sensitive")
#' 
#' # Inspect profile
#' plot(ancestral, chartType = "entroScan")
#'
#'
#' @export
#
getEntropySignature <- function(polymorphisms,
				position = "position",
				linkage = "linkage",
				ref = "ref",
				alt = "alt",
				protein = "protein",
				aa_position = "aa_position",
				ref_aa = "ref_aa",
				alt_aa = "alt_aa",
				alt_aa_freq = "alt_aa_freq",
			       	categories = "robust",
				genome = mn908947.3){
	#
	args <- as.list(environment())
	args$categories <- NULL
	processedData <- do.call(entropyProfile, args)
	perfil <- processedData$Perfil
	polymorphisms <-  processedData$Polymorphisms_multiplexed
	#
	# loop position-wise across SNVs
	for(posicion in unique(polymorphisms[,position])){
		thisPosition <- polymorphisms[polymorphisms[,position] == posicion,]
		# c. Check if in a coding region
		if(!is.na(thisPosition[,ref_aa][1])){# Affects a coding region
			# d. Check if non-synonymous
			matchesToReference <- sum(match(thisPosition[alt_aa], thisPosition[ref_aa]), na.rm = T)
			comparisons <- length(match(thisPosition[alt_aa], thisPosition[ref_aa]))
			if(matchesToReference - comparisons < 0){# at least one AA is different than reference AA.
				perfil$Entropy$entropy[perfil$Entropy$position == posicion] <- getPosEntropy(fillPosition(createPositionSummary(thisPosition, ref_aa, alt_aa, alt_aa_freq), categories))
			}
			else{ 
				perfil$Entropy$entropy[perfil$Entropy$position == posicion] <- 0
			}
		}
		else{# non coding region
			perfil$Entropy$entropy[perfil$Entropy$position == posicion] <- NA
		}
	}
	return(perfil)
}

---
title: "Overview of MetaEntropy"
bibliography: "overview.bib"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Overview of MetaEntropy}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

<br>

# Motivation
High-performance sequencing has exponentially increased the power
of intrapopulation viral diversity study.
Short-read methods are particularly popular due to their excellent
cost-to-depth ratio and the availability of efficient algorithms, such as the
Burrows-Wheeler Transform, to infer read homology within the framework of
reference genomes.

*MetaEntropy* is designed to process the typical output of variant calling
tools, consisting of information about the variants present in a population,
relative to a reference genome, and their corresponding frequencies.
It generates "informed" estimates of entropy at each genomic position, which,
in addition to allele frequencies, take into account the physicochemical
characteristics of the amino acids. The calculated values are
normalized to a scale of 0 to 1, facilitating the comparison of different loci
and different metagenomes.

<br>

# Input data

Single nucleotide variants (SNVs) data must be provided in a data frame.
This should contain columns indicating the reference genome positions
that are polymorphic in the virome, the linkage relationships between them, the
corresponding bases in the reference genome and in the virome, which protein and
amino acid position each mutation affects, the corresponding amino acids in the
reference strain and in the metagenome, and mutation frequencies.
For example, the *wWater* dataset, which accompanies the package, has this
information in the columns *position*, *linkage*, *ref*, *alt*, *protein*,
*aa_position*, *ref_aa*, *alt_aa* and *alt_aa_freq*, which are the labels
expected by default:


```{r loadMetaEntropy, results = "hide"}
library(MetaEntropy)
```
```{r seeLinked}
wWater[105:108,-1]
```

"Linked" mutations are variants that are physically connected in the genome and
affect a same codon.

There can also be positions with several different mutations:

```{r seewMultiple}
wWater[50:51,-1]
```

These variants collectively contribute to the entropy at the affected residue.

<br>

# Worked example: SARS-CoV-2 immune escape

## Loading packages

First we load some packages used in this example:

```{r loadPackages, results = "hide"}
lapply(c("ggplot2", "patchwork"), library, character.only = TRUE)
```

## Conceptual framework and data

This example uses data from wastewater colected during the first and third
COVID-19 waves in Argentina [BioProject PRJNA1183343; @manrique-jones2025].
The first wave was driven by ancestral SARS-CoV-2 lineages, while the third was
dominated by Omicron subvariants.
Omicron is a highly adapted lineage to humans, including hypervariation in the
receptor-binding domain (RBD) in response to immunity.

We will evidence the RBD relevance by an entropy analysis, and will identify
key mutations on it.

We begin by building separate datasets for ancestral and Omicron lineages.

```{r datasets}
firstWave <- wWater[ wWater$wave == "first", ]
thirdWave <- wWater[ wWater$wave == "third", ]
```
A 3% minor variant cutoff was already applied to these data by the variant
caller.
Following the same logic, here we will also filter out minor variants whose
genotype matches that of the reference genome:

```{r filter}
firstWave <- firstWave[ firstWave$alt_aa_freq <= 0.97, ]
thirdWave <- thirdWave[ thirdWave$alt_aa_freq <= 0.97, ]
```


## Entropy signatures

Now we infer the ancestral and Omicron signatures and compare them graphically.

```{r signatures}
ancestral <- getEntropySignature(firstWave)
omicron <-  getEntropySignature(thirdWave)
# Compare signatures
anc_plot <- plot(ancestral) + ggtitle("Ancestral")
omi_plot <- plot(omicron) + ggtitle("Omicron")
anc_plot / omi_plot
```

We see that entropy at the *S* gene is relatively high in the Omicron sublineages.


## Omicron sublineages in more detail

Now we look at how entropy is distributed along genome positions in the Omicron
sublineages.

```{r omicronEntroScan, fig.width = 7}
plot(omicron, chartType = "entroScan")
```

There are a handful of S positions that are responsible for the unusually high
entropy.

```{r}
omicron$Entropy$position[ omicron$Entropy$entropy > 0.3 ]
```

Let's look at the corresponding mutations:

```{r}
showMutations(omicron, c(22882, 22898, 22917, 23013, 23040, 23048, 23055, 23063))
```

The literature tells us that these SNVs affect the RBD [@Lan2020].

Viral RBDs are targeted by the immune system because they are
vital for cell binding and entry.
There is solid observational and experimental evidence for S:N440K, S:G446S,
S:L452R, S:E484A, S:Q493R, S:G496S, S:Q498R and S:N501Y [@Guigon2022;
@Liang2022; @Liu2021; @Motozono2022; @Rani2021; @Starr2021; @Starr2021a;
@Weisblum2020; @Zhang2022].

```{r, eval = FALSE}
# Search PUBMED for more studies on these mutations:
library(rentrez)
#
# create search phrase
mutations <- showMutations(omicron, c(22882, 22898, 22917, 23013, 23040, 23048, 23055, 23063))
searchPhrase <- paste0(gsub("S:", "", mutations$phenotype, fixed = TRUE), "[TIAB]")
searchPhrase <- paste0("SARS-CoV-2[TITLE] AND spike [TIAB] AND (", paste0(searchPhrase, collapse = " OR "), ")")
#
# search
mySearch <- entrez_search(db = "pubmed", term = searchPhrase, retmax = 1000)
#
# Number of studies
mySearch$count
[1] 560
#
```

We can do a formal analysis by **`assessHotSpot`()**.
The RBD is located between genomic positions 22517 and 23186.

```{r}
assessHotSpot(omicron, c(22517, 23186))
```

```{r}
assessHotSpot(ancestral, c(22517, 23186))
```

<br>


# Integrating Biological Signal via Entropy

Entropy analysis categorizes observations and takes into account the
corresponding frequencies in natural populations.
This adds significant information to sequence data.

```{r}
summary(ancestral)
```
In the summary above, we observe that *ORF6* and *N* proteins have high
mutational rates, both overall (SNV/Kb) and non-synonymous (non-Syn/Kb).
However, the corresponding entropies do not stand out among those of other
proteins.

This contrasts with *S*, which, despite having average mutation rates, exhibits
entropies biased towards high values.

This is even more evident for Omicron sublineages:

```{r}
summary(omicron)
```

<br>


# References

---
title: "`intraHostVariants`: SNVs from Multiple Anatomical Sites"
bibliography: "intraHostVariants.bib"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{`intraHostVariants`: SNVs from Multiple Anatomical Sites}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

Intra-host viral diversity (or *quasispecies*) drives resilience to the immune
responses and therapeutical interventions, while also fostering the emergence
of new lineages [@Andino2015; @Eigen1996; @Jones2025].
The `intraHostVariants` dataset contains SARS-CoV-2 single nucleotide variants
(SNVs) identified from autopsies of individuals with postmortem viral detection
and no documented immunocompromise [@Manrique2024].

Although `MetaEntropy` is tailored primarily for high-density wastewater data,
it is also effective for quantifying and analyzing intra-host viral protein
diversity, as demonstrated in this vignette.
For specialized or comprehensive quasispecies characterization, dedicated tools
such as `QSutils` [@GuerreroMurillo2018] are recommended.

First load the required packages:

```{r loadPackages, results = "hide"}
lapply(c("MetaEntropy", "ggplot2", "patchwork", "tidyr"),
       library, character.only = TRUE
)
```

# Data Structure and Characteristics

`intraHostVariants` is a `data.frame` containing information on 213 SNVs
inferred across lung, intestinal, cardiac, renal, and hepatic tissues.
Its 16 columns include everything required by `MetaEntropy`, alongside
experimental metadata and standard outputs typical of variant caller tables.
The underlying bioinformatic workflow is detailed in the `intraHostVariants`
documentation.

SARS-CoV-2 infections primarily cause respiratory illness, but the virus can
spread systemically and replicate independently across different anatomical
sites  [@Stein2022; @Manrique2024].
Consequently, individual tissues may harbor exclusive SNVs or share them at
varying frequencies.
This within-host diversity is well illustrated by `intraHostVariants`.

First, let's look at the distribution of SNVs across anatomical compartments:

```{r computeSNVs}
addmargins(
	   table(intraHostVariants$case,
		 intraHostVariants$organ
		 ),
	   margin = 2
)
```
Two to five SNVs are recorded per respiratory sample, closely aligning
with ranges reported in prior swab-based studies [@Jones2025].
However, the total number of SNVs recorded per individual ranges from 10 to 88
across all tissues.

Next, let's examine the distribution of SNV frequencies.
First, we pivot the dataset to place the alternative amino acid frequencies
into separate columns for each anatomical compartment:

```{r widenData, results = "hide"}
wide_ihv <- tidyr::pivot_wider(intraHostVariants,
		 names_from = organ,
		 values_from = alt_aa_freq,
		 # Use 0 (instead of NA) for undetected
		 # mutations.
		 values_fill = list(alt_aa_freq = 0)
)
```

We can now use `wide_ihv` to visualize these variant frequencies across tissues with
parallel coordinates plots:

```{r doParCoordPlot, fig.width = 4, fig.height = 8, out.width = "50%", results = "hide"}
par(mfcol = c(4,1), mar = c(2, 4, 2, 2) + 0.1)
for(case in sort(unique(wide_ihv$case))){
	thisCase <- t(wide_ihv[wide_ihv$case == case, c(15, 16, 17, 18, 19)])
	matplot(thisCase, type = "b", pch = 19, col = rgb(0, 0, 0, 0.4), lty = 1,
		xaxt = "n", xlab = "",
		main = sub("c", "case ", case),
		ylab = "SNV frequency"
	)
	axis(side = 1, at = 1:5, labels = rownames(thisCase)
	)
}
```

This reveals that the viral quasispecies varies considerably between anatomical
sites.
For instance, the lung and heart in case 18 harbor three high-frequency SNVs
that are absent from other organs:

```{r c18Contrasts}
intraHostVariants[intraHostVariants$case == "c18" & intraHostVariants$alt_aa_freq > 0.2 ,
                  c(1, 2, 10, 13, 14, 15, 16)]
```

# Package Integration

Intra-host SNVs can be synonymous or non-synonymous, and the impact of
non-synonymous mutations varies depending on whether the resulting amino acid
substitution is conservative or radical.
From a virological perspective, these phenotypic changes are particularly
relevant because they are more likely to alter viral physiology.
`MetaEntropy` is particularly useful here because it calculates amino acid
entropy and allows residues to be classified based on their physicochemical
properties, offering a nuanced view of functional protein diversity.
We will now use `MetaEntropy` to evaluate amino acid diversity across
anatomical sites within each individual.

First, we split the dataset into separate `data.frame` objects according to the
experimental design strata (organ and case), storing them as a list:

```{r splitData, results = "hide"}
strata <- split(intraHostVariants,
                    list(intraHostVariants$organ, intraHostVariants$case),
                    sep = "_", drop = TRUE
)
```

## Create a custom `genome`

The SNVs of `intraHostVariants` were inferred using the genomic sequence of the
reference strain Wuhan-Hu-1, which is the same as that of the reference genome
provided with `MetaEntropy` (list object `mn908947.3`).
However, `mn908947.3` splits the nsp12 mature peptide into its ORF1a- and
ORF1b-derived segments (nucleotides 13,442--13,468 and 13,468--16,236,
respectively) to accommodate the -1 ribosomal frameshift.
In contrast, intraHostVariants does not use this split encoding; instead, nsp12
is represented as a single continuous block spanning nucleotides
13,442--16,236.
Therefore, the object passed to the `genome` argument of `getEntropySignature()`
must reflect this single continuous region.
We can modify mn908947.3 accordingly:


```{r createGenome, results = "hide"}
mn908947.3.ihv <- mn908947.3
# Get the nsp12 3' end and assign it to nsp12_end
nsp12_rows <- mn908947.3.ihv$CDS$protein %in% c("nsp12a", "nsp12b")
nsp12_end <- max(mn908947.3.ihv$CDS$end[nsp12_rows])
# Create an entry for nsp12 from nsp12a
mn908947.3.ihv$CDS$protein[mn908947.3.ihv$CDS$protein == "nsp12a"] <- "nsp12"
# Update the 3' end
mn908947.3.ihv$CDS$end[mn908947.3.ihv$CDS$protein == "nsp12"] <- nsp12_end
# dismiss the old nsp12b annotation
mn908947.3.ihv$CDS <- mn908947.3.ihv$CDS[mn908947.3.ihv$CDS$protein != "nsp12b", ]
# tidy up the environment
rm(nsp12_rows, nsp12_end)
```

## Entropy computation

Now we apply `getEntropySignature()` to each partition:

```{r computeEntropies, results = "hide"}
profiles <- lapply(strata, function(df) {
			   getEntropySignature(df, position = "POS", ref = "REF", alt = "ALT",
					       genome = mn908947.3.ihv
			   )
})
```

## Profiles comparison

Now we can visually compare entropy across anatomical sites using
`heatmap_entropyProfiles()`:

```{r doHeatMaps, fig.width = 9, fig.height = 7, out.width = "90%", results = "hide"}
heatmap_entropyProfiles(!!!profiles)
```

The heatmap indicates that total intra-host viral entropy extends beyond what
is observed in the respiratory tract alone.
We can summarize this difference across mature viral proteins by grouping
organs into respiratory versus non-respiratory compartments:


```{r doBoxPlots, fig.width = 9, fig.height = 7, out.width = "90%", results = "hide"}
combined_entropy <- lapply(profiles, function(p) p$Entropy) |> do.call(rbind, args = _)
combined_entropy <- cbind(combined_entropy,
                          system = factor(ifelse(grepl("lung", rownames(combined_entropy)),
                                                       "respiratory system", "other"),
                                          levels = c("respiratory system", "other")
                                   )
                    )
# Plot in genomic order
combined_entropy$protein <- factor(
                                   combined_entropy$protein,
                                   levels = mn908947.3.ihv$CDS$protein
)
bp <- ggplot2::ggplot(data = combined_entropy, aes(x = protein, y = entropy)) +
	ggplot2::geom_boxplot(varwidth = T) +
	ggplot2::facet_wrap(~ system, ncol = 1, scales = "free_y") +
	ggplot2::scale_x_discrete(drop = FALSE) +
	ggplot2::theme_bw() +
	ggplot2::theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
bp
```

Overall, this vignette demonstrates how `MetaEntropy` can untangle proteomic
variation across viral populations and subpopulations.
These insights offer directions for future research.
For instance, in the case of viral quasispecies shown here, the entropy
profiles highlight avenues to explore, such as how localized proteomic
variation influences immune responses, or the extent to which swab sampling captures
total intra-host viral diversity.

# References

---
title: "Overview of MetaEntropy"
bibliography: "overview.bib"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Overview of MetaEntropy}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

<br>

# Motivation
High-performance sequencing has exponentially increased the power
of intrapopulation viral diversity study.
Short-read methods are particularly popular due to their excellent
cost-to-depth ratio and the availability of efficient algorithms, such as the
Burrows-Wheeler Transform, to infer read homology within the framework of
reference genomes.

*MetaEntropy* is designed to process the typical output of variant calling
tools, consisting of information about the variants present in a population,
relative to a reference genome, and their corresponding frequencies.
It generates "informed" estimates of entropy at each genomic position, which,
in addition to allele frequencies, take into account the physicochemical
characteristics of the amino acids. The calculated values are
normalized to a scale of 0 to 1, facilitating the comparison of different loci
and different metagenomes.

<br>

# Input data

Single nucleotide variants (SNVs) data must be provided in a data frame.
This should contain columns indicating the reference genome positions
that are polymorphic in the virome, the linkage relationships between them, the
corresponding bases in the reference genome and in the virome, which protein and
amino acid position each mutation affects, the corresponding amino acids in the
reference strain and in the metagenome, and mutation frequencies.
For example, the *wWater* dataset, which accompanies the package, has this
information in the columns *position*, *linkage*, *ref*, *alt*, *protein*,
*aa_position*, *ref_aa*, *alt_aa* and *alt_aa_freq*, which are the labels
expected by default:


```{r loadMetaEntropy, results = "hide"}
library(MetaEntropy)
```
```{r seeLinked}
wWater[105:108,-1]
```

"Linked" mutations are variants that are physically connected in the genome and
affect a same codon.

There can also be positions with several different mutations:

```{r seewMultiple}
wWater[50:51,-1]
```

These variants collectively contribute to the entropy at the affected residue.

<br>

# Worked example: SARS-CoV-2 immune escape

## Loading packages

First we load some packages used in this example:

```{r loadPackages, results = "hide"}
lapply(c("ggplot2", "patchwork"), library, character.only = TRUE)
```

## Conceptual framework and data

This example uses data from wastewater colected during the first and third
COVID-19 waves in Argentina [BioProject PRJNA1183343; @Manrique2026].
The first wave was driven by ancestral SARS-CoV-2 lineages, while the third was
dominated by Omicron subvariants.
Omicron is a highly adapted lineage to humans, including hypervariation in the
receptor-binding domain (RBD) in response to immunity.

We will evidence the RBD relevance by an entropy analysis, and will identify
key mutations on it.

We begin by building separate datasets for ancestral and Omicron lineages.

```{r datasets}
firstWave <- wWater[ wWater$wave == "first", ]
thirdWave <- wWater[ wWater$wave == "third", ]
```
A 3% minor variant cutoff was already applied to these data by the variant
caller.
Following the same logic, here we will also filter out minor variants whose
genotype matches that of the reference genome:

```{r filter}
firstWave <- firstWave[ firstWave$alt_aa_freq <= 0.97, ]
thirdWave <- thirdWave[ thirdWave$alt_aa_freq <= 0.97, ]
```


## Entropy signatures

Now we infer the ancestral and Omicron signatures and compare them graphically.

```{r signatures}
ancestral <- getEntropySignature(firstWave)
omicron <-  getEntropySignature(thirdWave)
# Compare signatures
anc_plot <- plot(ancestral) + ggtitle("Ancestral")
omi_plot <- plot(omicron) + ggtitle("Omicron")
anc_plot / omi_plot
```

We see that entropy at the *S* gene is relatively high in the Omicron sublineages.


## Omicron sublineages in more detail

Now we look at how entropy is distributed along genome positions in the Omicron
sublineages.

```{r omicronEntroScan, fig.width = 7}
plot(omicron, chartType = "entroScan")
```

There are a handful of S positions that are responsible for the unusually high
entropy.

```{r}
omicron$Entropy$position[ omicron$Entropy$entropy > 0.3 ]
```

Let's look at the corresponding mutations:

```{r}
showMutations(omicron, c(22882, 22898, 22917, 23013, 23040, 23048, 23055, 23063))
```

The literature tells us that these SNVs affect the RBD [@Lan2020].

Viral RBDs are targeted by the immune system because they are
vital for cell binding and entry.
There is solid observational and experimental evidence for S:N440K, S:G446S,
S:L452R, S:E484A, S:Q493R, S:G496S, S:Q498R and S:N501Y [@Guigon2022;
@Liang2022; @Liu2021; @Motozono2022; @Rani2021; @Starr2021; @Starr2021a;
@Weisblum2020; @Zhang2022].

```{r, eval = FALSE}
# Search PUBMED for more studies on these mutations:
library(rentrez)
#
# create search phrase
mutations <- showMutations(omicron, c(22882, 22898, 22917, 23013, 23040, 23048, 23055, 23063))
searchPhrase <- paste0(gsub("S:", "", mutations$phenotype, fixed = TRUE), "[TIAB]")
searchPhrase <- paste0("SARS-CoV-2[TITLE] AND spike [TIAB] AND (", paste0(searchPhrase, collapse = " OR "), ")")
#
# search
mySearch <- entrez_search(db = "pubmed", term = searchPhrase, retmax = 1000)
#
# Number of studies
mySearch$count
[1] 560
#
```

We can do a formal analysis by **`assessHotSpot`()**.
The RBD is located between genomic positions 22517 and 23186.

```{r}
assessHotSpot(omicron, c(22517, 23186))
```

```{r}
assessHotSpot(ancestral, c(22517, 23186))
```

<br>


# Integrating Biological Signal via Entropy

Entropy analysis categorizes observations and takes into account the
corresponding frequencies in natural populations.
This adds significant information to sequence data.

```{r}
summary(ancestral)
```
In the summary above, we observe that *ORF6* and *N* proteins have high
mutational rates, both overall (SNV/Kb) and non-synonymous (non-Syn/Kb).
However, the corresponding entropies do not stand out among those of other
proteins.

This contrasts with *S*, which, despite having average mutation rates, exhibits
entropies biased towards high values.

This is even more evident for Omicron sublineages:

```{r}
summary(omicron)
```

<br>


# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropyProfile.R
\name{entropyProfile}
\alias{entropyProfile}
\title{Create (empty) object of class "entropyProfile"}
\usage{
entropyProfile(
  polymorphisms,
  position = "position",
  linkage = "linkage",
  ref = "ref",
  alt = "alt",
  protein = "protein",
  aa_position = "aa_position",
  ref_aa = "ref_aa",
  alt_aa = "alt_aa",
  alt_aa_freq = "alt_aa_freq",
  entropies = NA_real_,
  genome = mn908947.3
)
}
\arguments{
\item{polymorphisms}{A data frame. Please see Details and Examples in
documentation for \code{\link{getEntropySignature}}.}

\item{position}{Name of the \code{polymorphisms}'s column that indicates SNV
locations in the genome.}

\item{linkage}{Information on linked positions.}

\item{ref}{Column name with reference bases.}

\item{alt}{Column name with the alternative bases observed in the
metagenome.}

\item{protein}{Name of the column carrying protein names.}

\item{aa_position}{Name of the column that indicates the protein positions
of the mutated amino acids.}

\item{ref_aa}{Name of the column that carries the reference amino acids.}

\item{alt_aa}{Name of the column carrying alternative amino acids observed
in the metagenome.}

\item{alt_aa_freq}{Name of the column giving the frequencies of alternative
amino acids in the metagenome.}

\item{entropies}{\code{NA_REAL_} (double numeric/real vector to hold entropy
values).}

\item{genome}{A list providing CDS data and length of the reference genome.}
}
\value{
An (empty) object of class \code{entropyProfile}.
}
\description{
This function is intended primarily for internal use by
\code{\link{getEntropySignature}}.
}
\details{
The documentation for \code{\link{getEntropySignature}} details the type of
input needed to create a profile. \code{entropyProfile} uses the same parameters as
\code{getEntropySignature}, with the exception of \code{categories} and
\code{entropies}.
}
\seealso{
\code{\link{getEntropySignature}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessHotSpot.R
\name{assessHotSpot}
\alias{assessHotSpot}
\title{Evaluates Entropy Hotspot}
\usage{
assessHotSpot(profile, boundaries, chartType = "boxplot")
}
\arguments{
\item{profile}{An object of class \code{entropyProfile}.}

\item{boundaries}{Numeric vector with the first and last genomic positions
of the region to be evaluated. To be set interactively if not
provided.}

\item{chartType}{Chart type; either "boxplot", "stripchart" or "swarm".}
}
\value{
\code{htest} object. This function is called primarily for its side
effects.
}
\description{
Graphical and formal analyses of contiguous amino acids.
}
\details{
The query stretch (\emph{e.g.} a protein domain with neutralizing
epitopes) is compared against the full set of proteins. Hot spot
boundaries should be indicated relative to the reference genome
used in variant calling.
}
\examples{
omicron <- getEntropySignature(wWater[wWater$wave == "third", ])

# Entrpy hotspot at SARS-CoV-2 receptor binding domain
assessHotSpot(omicron, c(22517, 23186), chartType = "swarm")

}
\seealso{
\code{\link{getEntropySignature}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPosEntropy.R
\name{getPosEntropy}
\alias{getPosEntropy}
\title{Calculates aminoacidic entropy resulting from SNVs present in a specific locus}
\usage{
getPosEntropy(variantPosition)
}
\arguments{
\item{variantPosition}{A list, created  and passed to the function by
\code{fillPosition}. It contains information on the
frequencies of each amino acid category observed in a
virome, as a result of mutations at a given genomic
position.}
}
\value{
A \code{numeric} value, corresponding to the entropy associated to
the SNVs present at a specific locus.
}
\description{
The function is used internally by \code{\link{getEntropySignature}}.
It calculates Shannon entropy from categories and frequencies passed to the
function.
}
\seealso{
\code{\link{getEntropySignature}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatMap_entropyProfiles.R
\name{heatmap_entropyProfiles}
\alias{heatmap_entropyProfiles}
\title{Graphically compare entropy profiles}
\usage{
heatmap_entropyProfiles(..., keepInvariant = TRUE, showAllPeptides = TRUE)
}
\arguments{
\item{...}{Unquoted \code{entropyProfile} data objects to be compared.}

\item{keepInvariant}{Logical; whether to retain positions where entropy is
zero. Defaults to \code{FALSE}.}

\item{showAllPeptides}{Logical; whether the heatmap should show all peptides
encoded by the virus, regardless of whether they have undergone mutations.
Defaults to \code{TRUE}.}
}
\value{
A \code{ggplot} object representing the heatmap.
}
\description{
This function prints heatmaps of multiple entropy profiles
(\code{entropyProfile} objects).
}
\examples{
firstWave <- wWater[ wWater$wave == "first", ]
thirdWave <- wWater[ wWater$wave == "third", ]
# Filter out minor variants whose genotype matches that of the reference genome
firstWave <- firstWave[ firstWave$alt_aa_freq <= 0.97, ]
thirdWave <- thirdWave[ thirdWave$alt_aa_freq <= 0.97, ]
ancestral <- getEntropySignature(firstWave)
omicron <-  getEntropySignature(thirdWave)

# High entropies sustained over time in the spike protein, and signals
# compatible with pervasive negative selection in Omicron sublineages:
heatmap_entropyProfiles(ancestral, omicron)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.entropyProfile.R
\name{summary.entropyProfile}
\alias{summary.entropyProfile}
\title{Summarize entropy profile}
\usage{
\method{summary}{entropyProfile}(object, ...)
}
\arguments{
\item{object}{An object of class \code{entropyProfile}.}

\item{...}{Other parameters passed to the function.}
}
\value{
An object of class \code{c("profileSummary", "list")} summarizing
an entropy profile. Intended to be displayed via
\code{print.profileSummary}.
}
\description{
Prints a report about an entropy profile (an object of class
"entropyProfile").
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.entropyProfile.R
\name{plot.entropyProfile}
\alias{plot.entropyProfile}
\title{Plot entropy signatures}
\usage{
\method{plot}{entropyProfile}(x, chartType = "bp", ...)
}
\arguments{
\item{x}{Object of class \code{entropyProfile}.}

\item{chartType}{Whether to graph per-protein summaries ("bp"), per-protein
stripcharts ("stripchart" / "swarm"), or position-wise
entropy ("entroScan").}

\item{...}{Additional arguments passed to the function.}
}
\value{
Unrendered \code{gg/ggplot} object produced by \code{ggplot2}. This
function is primarily called for its side effects.
}
\description{
Creates entropy charts along a genome.
}
\examples{
ancestral <- getEntropySignature(wWater[wWater$wave == "first", ])
omicron <- getEntropySignature(wWater[wWater$wave == "third", ])

# Enhanced Spike entropy plus pervasive negative selection in Omicron
# sublineages
anc_plot <- plot(ancestral, chartType = "stripchart")
omi_plot <- plot(omicron, chartType = "stripchart")
patchwork::wrap_plots(anc_plot/omi_plot)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intraHostVariants.R
\docType{data}
\name{intraHostVariants}
\alias{intraHostVariants}
\title{SNVs from multiple organs of individuals with postmortem SARS-CoV-2 detection}
\format{
A data frame with 213 rows and 16 columns.
}
\source{
Manrique, J. M., Maffia-Bizzozero, S., Delpino, M. V., Quarleri, J., &
Jones, L. R. (2024). Multi-Organ Spread and Intra-Host Diversity of
SARS-CoV-2 Support Viral Persistence, Adaptation, and a Mechanism That
Increases Evolvability. \emph{Journal of Medical Virology}, 96(12), e70107.
\doi{10.1002/jmv.70107}
}
\usage{
intraHostVariants
}
\description{
SNVs were inferred from Illumina (2 x 150) sequences. Reads were aligned to
the Wuhan-Hu-1 reference genome (MN908947.3) with \code{BWA}, followed by
variant calling using \code{LoFreq} with source quality correction. The VCF
file was post-processed with \code{BCFtools} to prevent reference and strand
biases and remove positions supported by fewer than 1,000 reads.
}
\seealso{
See \code{vignette("intraHostVariants", package = "MetaEntropy")} for a
step-by-step tutorial on analyzing this dataset.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createStorage.R
\name{createStorage}
\alias{createStorage}
\title{Build a structure representing amino acid categories.}
\usage{
createStorage(categories)
}
\arguments{
\item{categories}{A character string. Similar to the \code{categories}
parameter of \code{\link{getEntropySignature}}.}
}
\value{
A list with one element (\code{character} vector) for amino acid
category and an element (empty named \code{numeric} vector) to be
loaded with the frequency in the metagenome of each amino acid category.
}
\description{
The function is used internally by \code{\link{fillPosition}}, which in
turn is an auxiliary function of \code{\link{getEntropySignature}}.
It creates a list with one element for each amino acid category, named
according to the categories used (e.g., "aliphatic", "aromatic", etc.).
Each element contains a set of amino acids identified by one-letter codes.
The list also includes an element containing an empty numeric vector, whose
names correspond to the labels of each category.
This vector is to be populated with the frequency of each category at a
given genomic position, by the \code{fillPosition} function.
}
\seealso{
\code{\link{getEntropySignature}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createPositionSummary.R
\name{createPositionSummary}
\alias{createPositionSummary}
\title{Summarize variants and frequencies at a genome position}
\usage{
createPositionSummary(variants, ref_aa, alt_aa, alt_aa_freq)
}
\arguments{
\item{variants}{A data frame, similar to the \code{polymorphisms} argument
of \code{\link{getEntropySignature}}, but containing
information on a single genome position.}

\item{ref_aa}{Name of the column that carries reference amino acids.}

\item{alt_aa}{Name of the column carrying alternative amino acids observed
in the metagenome.}

\item{alt_aa_freq}{Name of the column giving the frequencies of alternative
amino acids.}
}
\value{
A \code{data frame} describing the variability (different amino
acids its frequencies) observed at a specific locus.
}
\description{
This function is used internally by \code{getEntropySignature()}.
It creates a vector (\code{aminoAcids}) listing the amino acids
observed in a virome at a particular position under analysis, including the
reference amino acid, another vector (\code{frequencies}) with the
corresponding frequencies, and returns them combined in a data frame.
}
\seealso{
\code{\link{getEntropySignature}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/showMutations.R
\name{showMutations}
\alias{showMutations}
\title{Summarize mutations}
\usage{
showMutations(profile, positions)
}
\arguments{
\item{profile}{An object of class \code{entropyProfile}.}

\item{positions}{A vector with genome positions relative to the reference
genome.}
}
\value{
An object of class \code{c("tidyMutations", "data.frame")},
containing summary information about user-supplied genomic
positions. This information includes the mutations themselves
relative to the reference genome, their positions within it, and the
corresponding abundances in the virome. Intended to be displayed by
\code{print.tidyMutations}.
}
\description{
Displays SNVs, and corresponding protein mutations, at specific genomic
positions.
}
\details{
The user provides a list of genome positions and the function
prints the mutations associated with them.
The output format is "ref_res###alt_res /
protein:ref_res###alt_res", where ref_res is the residue (eiter
nucleotide or aminoacid) in the reference strain, alt_res is the
alternative residue in the metagenome, "###" is the position
(either nucleotide or aminoacid) where the mutation was observed,
and "protein" is the name of the affected protein.
}
\examples{

# High entropy at the RBD in Omicron lineages
omicron <- getEntropySignature(wWater[wWater$wave == "third", ])
plot(omicron, chartType="stripchart")

# Identify the high-entropy positions
omicron$Entropy$position[ omicron$Entropy$entropy > 0.3 ]
#[1] 22882 22898 22917 23013 23040 23048 23055 23063

# Get a descriptive table
showMutations(omicron, c(22882, 22898, 22917, 23013, 23040, 23048, 23055, 23063))


}
\seealso{
\code{\link{getEntropySignature}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.entropyProfile.R
\name{as.data.frame.entropyProfile}
\alias{as.data.frame.entropyProfile}
\title{Coerce \code{entropyProfile} to a Data Frame}
\usage{
\method{as.data.frame}{entropyProfile}(x, row.names = NULL, optional = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{entropyProfile}.}

\item{row.names}{Please see \code{\link{as.data.frame}}.}

\item{optional}{Please see \code{\link{as.data.frame}}.}

\item{...}{Additional arguments passed to the function.}
}
\value{
A \code{data frame} with tabular information on an entropy profile.
This information includes the name of the proteins presenting
mutations, the corresponding genomic positions, and the resulting
entropies in the metagenome.
}
\description{
Function to extract summary information from an \code{entropyProfile}
object. This function is internally used for plotting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.profileSummary.R
\name{print.profileSummary}
\alias{print.profileSummary}
\title{Print method for  \code{profileSummary} objects}
\usage{
\method{print}{profileSummary}(x, ...)
}
\arguments{
\item{x}{An object of class \code{profileSummary} created by
\code{\link{summary.entropyProfile}}.}

\item{...}{Additional arguments passed to the function.}
}
\value{
Invisibly returns \code{NULL}. This function is used for its side
effect.
}
\description{
This function formats and prints compact entropy profile summaries
(\code{profileSummary} objects) created by \code{summary.entropyProfile()},
on the console.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mn908947.3.R
\docType{data}
\name{mn908947.3}
\alias{mn908947.3}
\title{CDS topology and length of Wuhan-Hu-1 reference strain}
\format{
An object of class \code{list} of length 2.
}
\source{
Nucleotide [Internet]. Bethesda (MD): National Library of Medicine (US),
National Center for Biotechnology Information; [1988] – . Accession No.
MN908947.3, Severe acute respiratory syndrome coronavirus 2 isolate Wuhan-Hu-1, complete genome.
Available from: \url{https://www.ncbi.nlm.nih.gov/nuccore/MN908947.3}.
}
\usage{
mn908947.3
}
\description{
This type of data can be obtained from \code{.gff} (General Feature Format)
files using applications such as the \code{rtracklayer} package, or manually
from the corresponding entry in the GenBank database.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wWater.R
\docType{data}
\name{wWater}
\alias{wWater}
\title{Data from first and third COVID-19 waves in Trelew \url{http://tools.wmflabs.org/geohack/geohack.php?language=es&pagename=Trelew&params=-43.253333333333_N_-65.309444444444_E_type:city}}
\format{
An object of class \code{data.frame} with 148 rows and 10 columns.
}
\source{
Manrique, Julieta Marina, and Leandro Roberto Jones. 2026. Genomic
surveillance of SARS-CoV-2: a resource-efficient wastewater-based
workflow. Molecular Biology Reports 53 : 995.
}
\usage{
wWater
}
\description{
SNVs inferred from Illumina (2 x 150) sequences from pooled ultra-pure virus
concentrates representative of the 1st and 3rd COVID-19 waves in Trelew.
Reads were mapped against the Wuhan-Wu-1 reference genome (MN908947.3) by
\emph{bwa}, and variants were called with \emph{iVar} with a 3\% frequency
cutoff for minor variants.
First wave cases were caused by ancestral strains whereas third wave cases
were mainly due to highly human-adapted Omicron sublineages.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.tydyMutations.R
\name{print.tidyMutations}
\alias{print.tidyMutations}
\title{Print method for  \code{tidyMutations} objects}
\usage{
\method{print}{tidyMutations}(x, ...)
}
\arguments{
\item{x}{An object of class \code{tidyMutations} created by
\code{\link{showMutations}}.}

\item{...}{Additional arguments passed to the function.}
}
\value{
Invisibly returns \code{NULL}. Called for side effect.
}
\description{
This function formats and prints compact mutation summaries
(\code{tidyMutations} objects) created by showMutations(), on the console.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getEntropySignature.R
\name{getEntropySignature}
\alias{getEntropySignature}
\title{Infer Entropy Signature}
\usage{
getEntropySignature(
  polymorphisms,
  position = "position",
  linkage = "linkage",
  ref = "ref",
  alt = "alt",
  protein = "protein",
  aa_position = "aa_position",
  ref_aa = "ref_aa",
  alt_aa = "alt_aa",
  alt_aa_freq = "alt_aa_freq",
  categories = "robust",
  genome = mn908947.3
)
}
\arguments{
\item{polymorphisms}{A data frame. Please see Details and Examples.}

\item{position}{Name of the \code{polymorphisms}'s column that indicates SNV
locations in the genome.}

\item{linkage}{Information on linked positions.}

\item{ref}{Column name with reference bases.}

\item{alt}{Column name with the alternative bases observed in the
metagenome.}

\item{protein}{Name of the column carrying protein names.}

\item{aa_position}{Name of the column that indicates the protein positions
of the mutated amino acids.}

\item{ref_aa}{Name of the column that carries the reference amino acids.}

\item{alt_aa}{Name of the column carrying alternative amino acids observed
in the metagenome.}

\item{alt_aa_freq}{Name of the column giving the frequencies of alternative
amino acids in the metagenome.}

\item{categories}{Whether a class per amino acid should be used
("sensitive") or they should be grouped into aliphatic, aromatic,
polar, positively charged, negatively charged, and special ("robust")
(Mirny and Shakhnovich, 1999).}

\item{genome}{A list providing CDS data and length of the reference genome.}
}
\value{
An object of class \code{entropyProfile}. It contains a tidy,
summarized version of the SNV table, a \code{data frame} with
information on genome-wide entropy, a \code{data frame} with
information on each CDS and corresponding mutations observed in the
virome, and a \code{list} with CDS data and length of the reference
genome used in variant calling.
}
\description{
Calculates genome-wide Shannon entropies from SNV data.
}
\details{
You provide a data frame with SNVs information including reference
and alternative aminoacids, their frequencies, and corresponding positions
relative to a reference sequence.
This type of data can be generated by numerous programs and pipelines.
The objective is to assess the biological impact of nonsynonymous
variation within a viral population, such as an environmental sample (e.g.
wastewater) or a single infection (aka quasisepecies).
Entropy is calculated \emph{within} the metagenome and is therefore independent
of the reference sequence.
Some mutations may be part of a same codon.
This is to be indicated in the \code{linkage} column, providing a downstream
linked position, or the closest upstream position if there are no downstream
positions that are part of the same codon.
For example, in the \code{wWater} dataset, mutations T22673C and C22674T are linked
to each other and affect codon 371 of the S gene:
\tabular{lccccccc}{
\tab	wave	\tab	position	\tab	linkage	\tab	ref	\tab	alt	\tab	protein	\tab ... \cr
...	\tab		\tab			\tab		\tab		\tab		\tab		\tab     \cr
105	\tab	third	\tab	22599		\tab	NA	\tab	G	\tab	A	\tab	S	\tab ... \cr
106	\tab	third	\tab	22673		\tab	22674	\tab	T	\tab	C	\tab	S	\tab ... \cr
107	\tab	third	\tab	22674		\tab	22673	\tab	C	\tab	T	\tab	S	\tab ... \cr
108	\tab	third	\tab	22679		\tab	NA	\tab	T	\tab	C	\tab	S	\tab ... \cr
...	\tab		\tab			\tab		\tab		\tab     \cr
}

The \code{genome} parameter is a list that provides data on the topology of
protein-coding regions in the genome and its length, used internally
primarily for graphical and summary purposes.
The package provides an example (\code{\link{mn908947.3}}) of how  this
information is to be organized.
}
\examples{

# Entropy across the genome in ancestral lineages
ancestral <- getEntropySignature(wWater[wWater$wave == "first", ], categories = "sensitive")

# Inspect profile
plot(ancestral, chartType = "entroScan")


}
\references{
Mirny and Shakhnovich, 1999. J Mol Biol 291:177-196.
\doi{10.1006/jmbi.1999.2911}.

Shannon, 1948. Bell System Technical Journal, 27:379-423.
\doi{10.1002/j.1538-7305.1948.tb01338.x}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fillPosition.R
\name{fillPosition}
\alias{fillPosition}
\title{Translate amino acid frequencies into category frequencies}
\usage{
fillPosition(positionSummary, categories)
}
\arguments{
\item{positionSummary}{A data frame created by \code{\link{createPositionSummary}}.}

\item{categories}{A character string indicating which category scheme to
use. Similar to the \code{categories} parameter of
\code{\link{getEntropySignature}}.}
}
\value{
A list with information on the frequencies of each amino acid
category observed in a virome in a specific locus. The list contains
a \code{character} vector for each amino acid category, and a named
\code{numeric} vector containing the frequency of each category in
the metagenome.
}
\description{
The function is used internally by \code{\link{getEntropySignature}}.
It creates a storage list by \code{createStorage}, and loads on it the
frequency of each amino acid category based on the data contained in a data
frame passed to the function (\code{positionsSummary} parameter).
}
\seealso{
\code{\link{getEntropySignature}}.
}
